using Ewide.Core.Service;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace Ewide.Application.Service
{
    public interface IHouseMemberService
    {
        Task<dynamic> QueryMemberPageList([FromBody] UserInput input);
        Task AddUser(AddHouseMemberInput input);
        Task<dynamic> DefaultRole([FromQuery] string orgId);
        Task DeleteUser(DeleteUserInput input);
        Task UpdateUser(UpdateUserInput input);
        Task<dynamic> GetUser([FromQuery] QueryUserInput input);
        Task ChangeUserStatus(UpdateUserInput input);
        Task<dynamic> GetUserOwnRole([FromQuery] QueryUserInput input);
        Task GrantUserData(UpdateUserInput input);
        Task<dynamic> GetUserOwnData([FromQuery] QueryUserInput input);
        Task<dynamic> DefaultRoleRange();
    }
}
